#!/usr/bin/python3
# -*- coding: utf-8 -*-

### BEGIN LICENSE
 # Copyright (C) 2021 KylinSoft Co., Ltd.
 #
 # Authors:
 #  Yang Min yangmin@kylinos.cn
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import GLib

import os
import dbus
import signal
import dbus.service
import dbus.mainloop.glib

import subprocess

DBUSNAME = "com.kylin.screen.rotation"

class DemoException(dbus.DBusException):
    _dbus_error_name = "com.kylin.screen.rotation.Exception"

class TestObject(dbus.service.Object):
    INTERFACE = "com.kylin.screen.rotation.interface"
    def __init__(self, system_bus, mainloop):
        dbus.service.Object.__init__(self, system_bus,
                                     "/")
        self.mainloop = mainloop

        command = "find /sys/devices/platform/ -type f -name 'gsensor_info'"
        processGW = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell = True)
        self.ifaceGW = bytes.decode(processGW.stdout.read().strip())

    @dbus.service.method(INTERFACE,
                         in_signature = 's', out_signature = 'as')
    def HelloWorld(self, hello_message):
        print(str(hello_message))
        return ["Hello", "from example-service.py", "with unique name",
                bus.get_unique_name()]

    @dbus.service.method(INTERFACE,
                         in_signature = '', out_signature = 'ss')
    def GetTuple(self):
        return ("Hello Tuple", " from example-service.py")

    @dbus.service.method(INTERFACE,
                         in_signature = '', out_signature = 's')
    def GetCurrentScreenStatus(self):
        kernelStatus = ""
        if os.path.exists(self.ifaceGW):
            processGW2 = subprocess.Popen("cat " + self.ifaceGW, stdout=subprocess.PIPE, shell = True)
            result = bytes.decode(processGW2.stdout.read())
            gwx = gwy = gwz = -1
            for l in result.split("\n"):
                if ":" not in l:
                    continue
                if "axis_x" in l:
                    gwx = int(l.split(":")[1])
                if "axis_y" in l:
                    gwy = int(l.split(":")[1])
                if "axis_z" in l:
                    gwz = int(l.split(":")[1])
            if gwx >= 768 and gwx <= 1280:
                kernelStatus = "left"
            elif gwx >= 2816 and gwx <= 3328:
                kernelStatus = "right"
            else:
                kernelStatus = "normal"
        return kernelStatus 

    @dbus.service.method(INTERFACE,
                         in_signature = '', out_signature = '')
    def RaiseException(self):
        raise DemoException("The RaiseException method does what you might "
                            'expect')

    @dbus.service.method(INTERFACE,
                         in_signature = '', out_signature = '')
    def Exit(self):
        self.mainloop.quit()

if __name__ == '__main__':
    os.environ["TERM"] = "xterm"
    os.environ["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/X11R6/bin"

    dbus.mainloop.glib.DBusGMainLoop(set_as_default = True)
    mainloop = GLib.MainLoop()

    bus = dbus.SystemBus()
    name = dbus.service.BusName(DBUSNAME, bus)
    signal.signal(signal.SIGINT, lambda : mainloop.quit())
    obj = TestObject(bus, mainloop)

    print("Running kyas_system_service...\n")
    mainloop.run()
    print("Ending kyas_system_service...\n")
